/*
 * Copyright 2008,2009,2011,2012 Petr Machata
 *
 * This file is part of MicroSweeper.
 *
 * MicroSweeper is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MicroSweeper is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MicroSweeper.  If not, see <http://www.gnu.org/licenses/>.
 */

abstract class Field {
    protected final MineField mineField;  // parent
    public final byte fieldIdx;  // coordinate

    // Field icons.
    static public final int iconNone = 0;
    static public final int iconArrowRight = 1;
    static public final int iconArrowDown = 2;
    static public final int iconCircle = 3;

    protected Field(MineField mf, byte fieldIdx) {
	this.mineField = mf;
	this.fieldIdx = fieldIdx;
    }

    protected Field(MineField mf, Field copy) {
	this.mineField = mf;
	this.fieldIdx = copy.fieldIdx;
    }

    public int uncover() {
	return mineField.uncover(this);
    }

    public boolean uncovered() {
	return mineField.uncovered(this);
    }

    public void toggleFlag() {
	mineField.toggleFlag(this);
    }

    public boolean flagged() {
	return mineField.flagged(this);
    }

    public boolean hasMine() {
	return mineField.hasMine(this);
    }

    public boolean setMine(boolean state) {
	mineField.setMine(this, state);
	return true;
    }

    public ParticipantSelector getParticipantSelector() {
	return mineField.getParticipantSelector();
    }

    public void allParticipants(FieldEnumerator e) {
	ParticipantSelector parSel = getParticipantSelector();
	byte count = parSel.getCountParticipantIndices(mineField, fieldIdx);
	for (byte parIdx = 0; parIdx < count; ++parIdx) {
	    byte fieldIdx2 = parSel.getParticipant(mineField, fieldIdx, parIdx);
	    if (fieldIdx2 < 0)
		continue;
	    if (!e.field(mineField.getField(fieldIdx2)))
		break;
	}
    }

    public int countMines() {
	MineCounter b = new MineCounter();
	allParticipants(b);
	return b.mines;
    }

    public int getIcon() {
	return iconNone;
    }

    abstract public Field clone(MineField mf);
    abstract public Constraint getConstraint();
    abstract public String render();
    abstract public int getColor();
    abstract public boolean consistent();
}
