/*
 * Copyright 2008,2009,2010,2012 Petr Machata
 *
 * This file is part of MicroSweeper.
 *
 * MicroSweeper is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MicroSweeper is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MicroSweeper.  If not, see <http://www.gnu.org/licenses/>.
 */

import javax.microedition.lcdui.*;
import javax.microedition.midlet.*;

import javax.microedition.rms.RecordEnumeration;
import javax.microedition.rms.RecordStore;
import javax.microedition.rms.RecordStoreException;
import javax.microedition.rms.RecordListener;

import java.io.IOException;
import java.util.Vector;

public class MicroSweeper
    extends MIDlet
{
    private final MenuForm menuForm = new MenuForm(this);
    private MineSweeperCanvas activeGame = null;
    private final String storeName = "MicroSweeperStore";

    public MicroSweeper() {
	Display.getDisplay(this).setCurrent(menuForm);
	loadSettings();
    }

    private RecordStore openStore() {
	try {
	    RecordStore recordStore
		= RecordStore.openRecordStore(storeName, true);
	    return recordStore;
	}
	catch (RecordStoreException rsExc) {
	    System.out.println(rsExc);
	    return null;
	}
	catch (SecurityException secExc) {
	    System.out.println(secExc);
	    return null;
	}
    }

    private void loadSettings() {
	RecordStore recordStore = openStore();
	if (recordStore == null)
	    return;

	try {
	    RecordEnumeration recEnum
		= recordStore.enumerateRecords(null, null, false);
	    while(recEnum.hasNextElement() == true) {
		int recordId = recEnum.nextRecordId();
		byte[] data = recordStore.getRecord(recordId);
		try {
		    Setting setting = Setting.read(data);
		    menuForm.applySetting(setting);
		}
		catch (IOException e) {
		}
	    }
	    recEnum.destroy();
	}
	catch (RecordStoreException e) {
	    e.printStackTrace();
	}
    }

    private void saveSettings() {
	try {
	    RecordStore.deleteRecordStore(storeName);
	}
	catch (RecordStoreException e) {
	}

	Vector settings = menuForm.getSettings();
	RecordStore recordStore = openStore();
	if (recordStore == null)
	    return;

	// deleteRecordStore may fail, and indeed fails in emulator.
	// Delete the store manually item at a time.
	try {
	    RecordEnumeration recEnum
		= recordStore.enumerateRecords(null, null, false);
	    while(recEnum.hasNextElement() == true) {
		int recordId = recEnum.nextRecordId();
		recordStore.deleteRecord(recordId);
	    }
	    recEnum.destroy();
	}
	catch (RecordStoreException e) {
	    e.printStackTrace();
	}

	for (int i = 0; i < settings.size(); ++i) {
	    Setting setting = (Setting)settings.elementAt(i);
	    try {
		byte[] data = setting.render();
		recordStore.addRecord(data, 0, data.length);
	    }
	    catch (RecordStoreException e) {
		e.printStackTrace();
	    }
	    catch (IOException e) {
		e.printStackTrace();
	    }
	}
    }

    public void startApp() {
    }

    public void pauseApp() {
    }

    public void destroyApp(boolean unconditional) {
	saveSettings();
    }

    ////
    // Communication with menu...
    ////

    public void exitAction() {
	saveSettings();
	notifyDestroyed();
    }

    public void continueGame() {
	if (activeGame == null)
	    System.out.println("activeGame should be != null");
	else
	    Display.getDisplay(this).setCurrent(activeGame);
    }

    public void playAction() {
	MineField mf = menuForm.mineField(10, 10,
					  menuForm.wrapX(), menuForm.wrapY(),
					  menuForm.parSel(),
					  menuForm.sumsEnabled());

	MineFieldBuilder mfb = new RandomMineFieldBuilder
	    (mf, menuForm.howManyMines(), menuForm.acceptor());

	if (activeGame != null) {
	    activeGame.stop();
	    activeGame = null;
	}

	Display.getDisplay(this)
	    .setCurrent(new MineSweeperCanvas(this, mfb, menuForm.fullPlan()));
    }


    ////
    // Communication with game...
    ////

    public void exitGame() {
	activeGame = null;
	menuForm.haveActiveGame(false);
	Display.getDisplay(this).setCurrent(menuForm);
    }

    public void restartGame() {
	playAction();
    }

    public void showMenu(MineSweeperCanvas game) {
	activeGame = game;
	menuForm.haveActiveGame(true);
	Display.getDisplay(this).setCurrent(menuForm);
    }
}
