/*
 * Copyright 2008, 2012, Petr Machata
 *
 * This file is part of MicroSweeper.
 *
 * MicroSweeper is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MicroSweeper is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MicroSweeper.  If not, see <http://www.gnu.org/licenses/>.
 */

public abstract class MineFieldBuilder {
    private boolean initialized = false;
    final MineField mineField;

    /// Long-running builders may want to test for `done' and exit
    /// prematurely if it's set to true.
    protected boolean shouldStop = false;

    MineFieldBuilder(MineField mf) {
	this.mineField = mf;
    }

    public void stop() {
	shouldStop = true;
    }

    public boolean isInitialized() {
	return initialized;
    }

    public final byte initialize(Field f0)
	throws RuntimeException, // when double initialization
	       IllegalArgumentException // when idx1 outside of the board
    {
	if (initialized)
	    throw new RuntimeException("Double initialization.");

	byte ret = doInitialize(f0);
	initialized = true;
	return ret;
    }

    protected abstract byte doInitialize(Field f0);
    public abstract int getNumMines();

    public int getProgressWalls() { return 0; }
    public int getProgressMines() { return 0; }
}
