/*
 * Copyright 2009,2010,2011,2012 Petr Machata
 *
 * This file is part of MicroSweeper.
 *
 * MicroSweeper is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MicroSweeper is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MicroSweeper.  If not, see <http://www.gnu.org/licenses/>.
 */

class NormalField
    extends Field
{
    public NormalField(MineField mf, byte fieldIdx) {
	super(mf, fieldIdx);
    }

    public NormalField(MineField mf, NormalField copy) {
	super(mf, copy);
    }

    public Field clone(MineField mf) {
	return new NormalField(mf, this);
    }

    public char constraintCode() {
	return 'f';
    }

    public Constraint getConstraint() {
	class NeighboringConstraint extends Constraint {
	    NeighboringConstraint(final MineField mf, final byte fieldIdx) {
		super ("" + constraintCode() + fieldIdx);
		NormalField.this.allParticipants(new FieldEnumerator() {
			public boolean field(Field f) {
			    if (!f.uncovered())
				variables.add(f.fieldIdx);
			    return true;
			}
		    });
		result = NormalField.this.countMines();
	    }
	}

	return new NeighboringConstraint(mineField, fieldIdx);
    }

    public String render() {
	MineCounter b = new MineCounter();
	allParticipants(b);
	if (b.mines == 0
	    || ((b.total - b.uncovered) == b.mines && b.mines == b.flags))
	    return "";
	return Long.toString(b.mines);
    }

    public int getColor() {
	MineCounter b = new MineCounter();
	allParticipants(b);
	final int[] colors = {0,
			      0x000000ff, 0x0000ff00,
			      0x0000ffff, 0x00ff00ff,
			      0x00ffff00};
	if (b.mines == b.flags)
	    return 0;
	if (b.mines >= colors.length)
	    return 0x00ff0000;
	else
	    return colors[b.mines];
    }

    public boolean consistent() {
	MineCounter b = new MineCounter();
	allParticipants(b);
	return b.flags <= b.mines;
    }
}
