/*
 * Copyright 2010, Petr Machata
 *
 * This file is part of MicroSweeper.
 *
 * MicroSweeper is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MicroSweeper is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MicroSweeper.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

public class Setting
{
    final public String key;
    final public String value;

    public Setting(String key, String value) {
	this.key = key;
	this.value = value;
    }

    public static Setting read(byte[] data) throws IOException {
	ByteArrayInputStream is = new ByteArrayInputStream(data);
	DataInputStream dis = new DataInputStream(is);
	String key = dis.readUTF();
	String value = dis.readUTF();
	return new Setting(key, value);
    }

    public byte[] render() throws IOException {
	ByteArrayOutputStream os = new ByteArrayOutputStream();
	DataOutputStream dos = new DataOutputStream(os);
	dos.writeUTF(key);
	dos.writeUTF(value);
	dos.close();
	return os.toByteArray();
    }

    public String toString() {
	return "<" + key + "=" + value + ">";
    }
}
