/* -*-coding:utf-8-*-
 * Copyright 2008, 2011, Petr Machata
 *
 * This file is part of MicroSweeper.
 *
 * MicroSweeper is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MicroSweeper is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MicroSweeper.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.io.PrintStream;

class StringGridDisplay
    implements MineSweeperDisplay
{
    private boolean firstField = true;
    private final int gridW = 100;
    private final int gridH = 100;
    private int lastY = 0;
    private char[][] grid = new char[gridH][gridW];
    private final PrintStream out;
    private final boolean full;

    public StringGridDisplay(PrintStream out, boolean full) {
	this.out = out;
	this.full = full;
    }

    private void put(int y, int x, char c) {
	if (x < 0 || y < 0 || x >= gridW || y >= gridH)
	    return;
	if (y > lastY)
	    lastY = y;
	grid[y][x] = c;
    }

    private void paintField(Field f, int x, int y) {
	if (x < 0 || y < 0 || x >= gridW || y >= gridH)
	    return;

	char wallUp = ' ';//f.hasWall(Field.WALL_UP) ? '─' : ' ';
	char wallDown = ' ';//f.hasWall(Field.WALL_DOWN) ? '─' : ' ';

	String s;
	if (full) {
	    if (f.flagged())
		s = "F";
	    else if (!f.uncovered())
		s = "-";
	    else if (f.hasMine())
		s = "*";
	    else
		s = f.render();
	} else {
	    s = f.render();
	    if (f.hasMine())
		s += 'X';
	}

	while (s.length() < 2)
	    s = " " + s;

	put(y + 0, x + 0, '┌');
	put(y + 0, x + 1, wallUp);
	put(y + 0, x + 2, wallUp);
	put(y + 0, x + 3, '┐');

	put(y + 1, x + 0, /*f.hasWall(Field.WALL_LEFT) ? '│' :*/ ' ');
	put(y + 1, x + 1, s.charAt(0));
	put(y + 1, x + 2, s.charAt(1));
	put(y + 1, x + 3, /*f.hasWall(Field.WALL_RIGHT) ? '│' :*/ ' ');

	put(y + 2, x + 0, '└');
	put(y + 2, x + 1, wallDown);
	put(y + 2, x + 2, wallDown);
	put(y + 2, x + 3, '┘');
    }

    public void beforeDrawing() {}
    public void drawField(Field f, int fx, int fy, int fr) {
	// 4 and 3 are the size of out field template
	int xc = 4 * fx / fr;
	int yc = 3 * fy / fr;
	paintField(f, xc, yc);
    }

    public void afterDrawing() {
	// Patch leading zeroes where necessary
	for (int y = 0; y <= lastY; ++y)
	    for (int x = 0; x < gridW; ++x)
		if (grid[y][x] != '\0') {
		    for (int xx = 0; xx < x; ++xx)
			grid[y][xx] = ' ';
		    break;
		}

	// And print out the grid
	for (int i = 0; i <= lastY; ++i) {
	    int last = grid[i].length;
	    for (int j = 0; j < last; ++j)
		if (grid[i][j] == 0) {
		    last = j;
		    break;
		}

	    out.println(new String(grid[i], 0, last));
	}
	out.flush();
    }
}
