/*
 * Copyright 2008, Petr Machata
 *
 * This file is part of MicroSweeper.
 *
 * MicroSweeper is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MicroSweeper is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MicroSweeper.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.util.Vector;
import java.util.Enumeration;

abstract public class Timer {
    abstract public void enter();
    abstract public void leave();
    abstract public void dump();

    private static Vector registered = new Vector();

    static Timer createReg(String name) {
	Timer t = new NormalTimer(name);
	registered.addElement(t);
	return t;
    }

    static void dumpRegistered() {
	for (Enumeration e = registered.elements(); e.hasMoreElements() ;) {
	    Timer t = (Timer)e.nextElement();
	    t.dump();
	}
    }

    final public static Timer fake = new FakeTimer();
}

class NormalTimer extends Timer {

    private long enterTime = 0;
    private long timeAcc = 0;
    private int events = 0;
    private String name;

    public NormalTimer(String name) {
	this.name = name;
    }

    public void enter() {
	if (enterTime != 0)
	    throw new RuntimeException("enter after enter " + name);
	enterTime = System.currentTimeMillis();
    }

    public void leave() {
	long t = System.currentTimeMillis();
	timeAcc += t - enterTime;
	enterTime = 0;
	events += 1;
    }

    public void dump() {
	System.out.println(timeAcc + "\t" + name + "\t" + events);
    }
}

class FakeTimer extends Timer {
    public void enter() {}
    public void leave() {}
    public void dump() {}
}
