/* -*-java-*-
 * Copyright 2012 Petr Machata
 *
 * This file is part of MicroSweeper.
 *
 * MicroSweeper is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MicroSweeper is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MicroSweeper.  If not, see <http://www.gnu.org/licenses/>.
 */

class TotalCountField extends EmptyField {
    static private final ParticipantSelector parSelBoard
	= new ParticipantSelector() {
		public byte getCountParticipantIndices(MineField mf,
						       byte fieldIdx) {
		    return mf.getNumberOfFields();
		}

		public byte getParticipant(MineField mf,
					   byte fieldIdx, byte parIdx) {
		    return parIdx;
		}

		public int howManyMines(int level, int endLevel) {
		    return 0;
		}
	    };

    public TotalCountField(MineField mf, byte fieldIdx) {
	super(mf, fieldIdx);
    }

    public TotalCountField(MineField mf, TotalCountField copy) {
	super(mf, copy);
    }

    public String render() {
	MineCounter b = new MineCounter();
	allParticipants(b);
	if (b.mines == 0)
	    return "";
	int report = b.flags > b.mines ? b.mines : b.mines - b.flags;
	return Long.toString(report);
    }

    public Field clone(MineField mf) {
	return new TotalCountField(mf, this);
    }

    public char constraintCode() {
	return 'S';
    }

    public ParticipantSelector getParticipantSelector() {
	return parSelBoard;
    }

    public int getIcon() {
	return iconCircle;
    }

    public int getColor() {
	return 0;
    }
}
